@@ -155,7 +155,9 @@ class MultiBody {
    * Dimensionality of joint positions q (including 7-DoF floating-base
    * coordinates if this system is floating-base).
    */
-  TINY_INLINE int dof() const { return is_floating_ ? dof_q_ + 7 : dof_q_; }
+  TINY_INLINE int dof() const { 
+      return is_floating_ ? dof_q_ + 7 : dof_q_; 
+  }
   /**
    * Dimensionality of joint velocities qd and accelerations qdd (including
    * 6-DoF base velocity and acceleration, if this system is floating-base).
@@ -588,6 +590,7 @@ class MultiBody {
         "JOINT_FIXED",       "JOINT_PRISMATIC_X",    "JOINT_PRISMATIC_Y",
         "JOINT_PRISMATIC_Z", "JOINT_PRISMATIC_AXIS", "JOINT_REVOLUTE_X",
         "JOINT_REVOLUTE_Y",  "JOINT_REVOLUTE_Z",     "JOINT_REVOLUTE_AXIS",
+        "JOINT_SPHERICAL", "JOINT_INVALID",
     };
     return names[int(t) + 1];
   }
