@@ -115,12 +115,18 @@ struct MeshcatUrdfVisualizer {
   typedef ::tds::UrdfLink<Algebra> TinyUrdfLink;
   typedef ::tds::UrdfVisual<Algebra> UrdfVisual;
   
+  using Scalar = typename Algebra::Scalar;
   using Vector3 = typename Algebra::Vector3;
   using Quaternion = typename Algebra::Quaternion;
   using Matrix3x3 = typename Algebra::Matrix3;
   typedef tds::Transform<Algebra> Transform;
   typedef ::tds::MultiBody<Algebra> TinyMultiBody;
 
+  enum BASE_TRANSFORM_ORDER {
+      QUAT_XYZW_POSXYZ = 1,
+      POSXYZ_QUAT_XYZW,
+      };
+
   struct TinyVisualLinkInfo {
     std::string vis_name;
     int link_index;
@@ -358,6 +364,87 @@ struct MeshcatUrdfVisualizer {
       }
     }
   }
+
+    void sync_visual_transforms2(const TinyMultiBody* body, const std::vector<Scalar>& params, int link_visual_transforms_start_index, BASE_TRANSFORM_ORDER base_xform_layout = QUAT_XYZW_POSXYZ) {
+    // sync base transform
+    int index = link_visual_transforms_start_index;
+    for (int v = 0; v < body->visual_instance_uids().size(); v++) {
+      int visual_id = body->visual_instance_uids()[v];
+      if (m_b2vis.find(visual_id) != m_b2vis.end()) {
+        Vector3 pos(0,0,0);
+        Quaternion rot(0,0,0,1);
+        switch (base_xform_layout) { 
+            case POSXYZ_QUAT_XYZW:
+                pos = Vector3(params[0],params[1],params[2]);
+                rot = Quaternion (params[3],params[4],params[5],params[6]);
+                break;
+            case QUAT_XYZW_POSXYZ:
+                pos = Vector3(params[4],params[5],params[6]);
+                rot = Quaternion (params[0],params[1],params[2],params[3]);
+                break;
+            default: {
+                }
+         }
+        Transform base_world;
+        base_world.translation = pos;
+        base_world.rotation = Algebra::quat_to_matrix(rot);
+
+        Transform geom_X_world =
+            base_world * body->X_visuals()[v];
+
+        const Matrix3x3 &m =
+            geom_X_world.rotation;
+
+        const TinyVisualLinkInfo &viz = m_b2vis.at(visual_id);
+        // printf("vis_name=%s\n", viz.vis_name.c_str());
+        double world_pos[3] = {geom_X_world.translation.getX(),
+                               geom_X_world.translation.getY(),
+                               geom_X_world.translation.getZ()};
+        double world_mat[9] = {m.getRow(0)[0], m.getRow(1)[0], m.getRow(2)[0],
+                               m.getRow(0)[1], m.getRow(1)[1], m.getRow(2)[1],
+                               m.getRow(0)[2], m.getRow(1)[2], m.getRow(2)[2]};
+        nlohmann::json tr_cmd =
+            create_transform_cmd(world_pos, world_mat, viz.vis_name.c_str());
+        send_zmq(m_sock, tr_cmd);
+      }
+    }
+
+    for (int l = 0; l < body->links().size(); l++) {
+      for (int v = 0; v < body->links()[l].visual_instance_uids.size(); v++) {
+        int visual_id = body->links()[l].visual_instance_uids[v];
+        if (m_b2vis.find(visual_id) != m_b2vis.end()) {
+          
+          Vector3 pos (params[index],params[index+1],params[index+2]);
+          Quaternion rot(params[index+3],params[index+4],params[index+5],params[index+6]);
+          index+=7;
+          //Transform geom_X_world2a =
+          //    body->links()[l].X_world * body->links()[l].X_visuals[v];
+          Transform geom_X_world;
+          geom_X_world.translation = pos;
+          geom_X_world.rotation = Algebra::quat_to_matrix(rot);
+          
+          const Matrix3x3 &m =
+              geom_X_world.rotation;
+          const TinyVisualLinkInfo &viz = m_b2vis.at(visual_id);
+          // printf("vis_name=%s\n", viz.vis_name.c_str());
+          double world_mat[9] = {
+              m.getRow(0)[0], m.getRow(1)[0], m.getRow(2)[0],
+              m.getRow(0)[1], m.getRow(1)[1], m.getRow(2)[1],
+              m.getRow(0)[2], m.getRow(1)[2], m.getRow(2)[2]};
+          double world_pos[3] = {geom_X_world.translation.getX(),
+                                 geom_X_world.translation.getY(),
+                                 geom_X_world.translation.getZ()};
+          nlohmann::json tr_cmd =
+              create_transform_cmd(world_pos, world_mat, viz.vis_name.c_str());
+          send_zmq(m_sock, tr_cmd);
+        }
+      }
+    }
+  }
+
+  //no-op to be compatible
+  void render() {
+  }
 };
 
 #endif  // MESHCAT_URDF_VISUALIZER_H
